@@ -1,5 +1,5 @@
 // `error_chain!` can recurse deeply
-#![recursion_limit = "1024"]
+// #![recursion_limit = "1024"]
 #![allow(missing_docs)]
 /*!
 * This crate provides an interface into the Terra wallet service.
@@ -24,12 +24,12 @@
 /// Error Messages
 pub mod errors;
 
+use crate::errors::KeyringErrorAdapter;
 //#[macro_use]
-extern crate error_chain;
-
+//extern crate error_chain;
+use crate::errors::TerraRustWalletError;
 use bitcoin::secp256k1::All;
 use bitcoin::secp256k1::Secp256k1;
-use errors::Result;
 use serde::{Deserialize, Serialize};
 use terra_rust_api::{PrivateKey, PublicKey};
 
@@ -54,13 +54,15 @@ pub struct Wallet<'a> {
 impl<'a> Wallet<'a> {
     /// create a new wallet to store keys into. This just creates the structure
     /// use #new to create a new wallet
-    pub fn new(wallet_name: &'a str) -> Result<Wallet<'a>> {
+    pub fn new(wallet_name: &'a str) -> anyhow::Result<Wallet<'a>> {
         log::debug!("Creating new wallet {}", wallet_name);
         let wallet = Wallet::create(wallet_name);
         let wallet_list_name = &wallet.full_list_name();
         let keyring = keyring::Keyring::new(wallet_name, wallet_list_name);
         let wallet_internal = WalletInternal { keys: vec![] };
-        keyring.set_password(&serde_json::to_string(&wallet_internal)?)?;
+        keyring
+            .set_password(&serde_json::to_string(&wallet_internal)?)
+            .map_err(KeyringErrorAdapter::from)?;
         let string_key_name: String = String::from(wallet_name);
 
         match Wallet::get_wallets() {
@@ -96,10 +98,10 @@ impl<'a> Wallet<'a> {
         secp: &'a Secp256k1<All>,
         key_name: &'a str,
         seed: Option<&'a str>,
-    ) -> Result<PrivateKey> {
+    ) -> anyhow::Result<PrivateKey> {
         let full_key_name = self.full_key_name(key_name);
-        let keyring = keyring::Keyring::new(&self.name, &full_key_name);
-        let phrase = &keyring.get_password()?;
+        let keyring = keyring::Keyring::new(self.name, &full_key_name);
+        let phrase = &keyring.get_password().map_err(KeyringErrorAdapter::from)?;
 
         match seed {
             None => Ok(PrivateKey::from_words(secp, phrase)?),
@@ -112,18 +114,20 @@ impl<'a> Wallet<'a> {
         secp: &Secp256k1<All>,
         key_name: &str,
         seed: Option<&str>,
-    ) -> Result<PublicKey> {
+    ) -> anyhow::Result<PublicKey> {
         let private_key: PrivateKey = self.get_private_key(secp, key_name, seed)?;
 
-        let pub_key = private_key.public_key(&secp);
+        let pub_key = private_key.public_key(secp);
         Ok(pub_key)
     }
     /// stores the private key into the keyring
-    pub fn store_key(&self, key_name: &str, pk: &PrivateKey) -> Result<bool> {
+    pub fn store_key(&self, key_name: &str, pk: &PrivateKey) -> anyhow::Result<bool> {
         let full_key_name = self.full_key_name(key_name);
 
-        let keyring = keyring::Keyring::new(&self.name, &full_key_name);
-        keyring.set_password(pk.words().unwrap())?;
+        let keyring = keyring::Keyring::new(self.name, &full_key_name);
+        keyring
+            .set_password(pk.words().unwrap())
+            .map_err(KeyringErrorAdapter::from)?;
         let old_list = self.get_keys()?;
         let string_key_name: String = String::from(key_name);
         let mut new_list: Vec<String> = vec![];
@@ -140,10 +144,12 @@ impl<'a> Wallet<'a> {
         Ok(true)
     }
     /// deletes the private key from the keyring
-    pub fn delete_key(&self, key_name: &str) -> Result<bool> {
+    pub fn delete_key(&self, key_name: &str) -> anyhow::Result<bool> {
         let full_key_name = self.full_key_name(key_name);
-        let keyring = keyring::Keyring::new(&self.name, &full_key_name);
-        keyring.delete_password()?;
+        let keyring = keyring::Keyring::new(self.name, &full_key_name);
+        keyring
+            .delete_password()
+            .map_err(KeyringErrorAdapter::from)?;
         let old_list = self.get_keys()?;
         let mut new_list = vec![];
         for s in old_list {
@@ -156,20 +162,22 @@ impl<'a> Wallet<'a> {
         Ok(true)
     }
     /// lists the keys in the wallet
-    pub fn list(&self) -> Result<Vec<String>> {
+    pub fn list(&self) -> anyhow::Result<Vec<String>> {
         self.get_keys()
     }
 
     /// deletes the wallet and ALL the keys in the wallet
-    pub fn delete(&self) -> Result<()> {
+    pub fn delete(&self) -> anyhow::Result<()> {
         let keys = self.get_keys()?;
         for key in keys {
             log::debug!("Deleting Key {} in wallet {}", key, &self.name);
             self.delete_key(&key)?;
         }
         let wallet_list_name = self.full_list_name();
-        let keyring = keyring::Keyring::new(&self.name, &wallet_list_name);
-        keyring.delete_password()?;
+        let keyring = keyring::Keyring::new(self.name, &wallet_list_name);
+        keyring
+            .delete_password()
+            .map_err(KeyringErrorAdapter::from)?;
         let old_list = Wallet::get_wallets()?;
         // let string_key_name: String = String::from(self.name);
         let mut new_list: Vec<String> = vec![];
@@ -196,37 +204,48 @@ impl<'a> Wallet<'a> {
     }
 
     /// get list of keys in a wallet
-    fn get_keys(&self) -> Result<Vec<String>> {
+    fn get_keys(&self) -> anyhow::Result<Vec<String>> {
         let wallet_list_name = self.full_list_name();
-        let keyring = keyring::Keyring::new(&self.name, &wallet_list_name);
+        let keyring = keyring::Keyring::new(self.name, &wallet_list_name);
+        let pass = keyring
+            .get_password()
+            .map_err(|source| TerraRustWalletError::KeyNotFound {
+                key: wallet_list_name,
+                source: KeyringErrorAdapter::from(source),
+            })?;
 
-        let wallet_internal: WalletInternal = serde_json::from_str(&keyring.get_password()?)?;
+        let wallet_internal: WalletInternal = serde_json::from_str(&pass)?;
         Ok(wallet_internal.keys)
     }
 
     /// get list of wallets
-    pub fn get_wallets() -> Result<Vec<String>> {
+    pub fn get_wallets() -> anyhow::Result<Vec<String>> {
         let wallet_list_name = Wallet::wallet_list_name();
         let keyring = keyring::Keyring::new(&wallet_list_name, "wallets");
 
-        let wallet_internal: WalletListInternal = serde_json::from_str(&keyring.get_password()?)?;
+        let wallet_internal: WalletListInternal =
+            serde_json::from_str(&keyring.get_password().map_err(KeyringErrorAdapter::from)?)?;
         Ok(wallet_internal.wallets)
     }
 
     /// update keys in a wallet
-    fn set_keys(&self, int: &WalletInternal) -> Result<()> {
+    fn set_keys(&self, int: &WalletInternal) -> anyhow::Result<()> {
         let wallet_list_name = self.full_list_name();
-        let keyring = keyring::Keyring::new(&self.name, &wallet_list_name);
+        let keyring = keyring::Keyring::new(self.name, &wallet_list_name);
 
-        keyring.set_password(&serde_json::to_string(int)?)?;
+        keyring
+            .set_password(&serde_json::to_string(int)?)
+            .map_err(KeyringErrorAdapter::from)?;
         Ok(())
     }
     /// update list of wallets
-    fn set_wallets(int: &WalletListInternal) -> Result<()> {
+    fn set_wallets(int: &WalletListInternal) -> anyhow::Result<()> {
         let wallet_list_name = Wallet::wallet_list_name();
         let keyring = keyring::Keyring::new(&wallet_list_name, "wallets");
 
-        keyring.set_password(&serde_json::to_string(int)?)?;
+        keyring
+            .set_password(&serde_json::to_string(int)?)
+            .map_err(KeyringErrorAdapter::from)?;
         Ok(())
     }
 }
@@ -236,7 +255,7 @@ mod tst {
     use super::*;
 
     #[test]
-    pub fn test_wallet_create_delete() -> Result<()> {
+    pub fn test_wallet_create_delete() -> anyhow::Result<()> {
         let wallet = Wallet::new("PFC-Test Wallet")?;
         let key_list = wallet.get_keys()?;
         assert!(key_list.is_empty());
@@ -244,7 +263,7 @@ mod tst {
         Ok(())
     }
     #[test]
-    pub fn test_wallet_add_del() -> Result<()> {
+    pub fn test_wallet_add_del() -> anyhow::Result<()> {
         let str_1 = "notice oak worry limit wrap speak medal online prefer cluster roof addict wrist behave treat actual wasp year salad speed social layer crew genius";
         let str_2 = "wonder caution square unveil april art add hover spend smile proud admit modify old copper throw crew happy nature luggage reopen exhibit ordinary napkin";
 
